#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
skeppa
------------
:copyright: (c) 2016 Martin Sandström
:license: MIT, see LICENSE for more details.
"""


__title__ = "skeppa"
__version__ = "1.4.0"
__build__ = 140
__license__ = "MIT"
__copyright__ = "Copyright 2016 Martin Sandström"



import importlib
import os
import sys

import six
import yaml
from fabric.api import run
from fabric.context_managers import cd
from fabric.contrib.files import exists
from fabric.decorators import task
from fabric.state import env
from fabric.utils import abort
from jinja2 import Template

import ext
import settings as skeppa_settings
from api import build, deploy, ping, push, setup  # NOQA


def _create_stage(name, stage_config):
    def stage_wrap(*args, **kwargs):
        for key, value in stage_config.iteritems():
            setattr(env, key, value)

        env.exists = exists
        env.run = run
        env.cd = cd

    stage_wrap.__name__ = name
    return task(stage_wrap)


def _load_config(config_path):
    current_dir = os.getcwd()
    config_file = os.path.join(current_dir, config_path)

    with open(config_file, 'r') as stream:
        config_yaml = stream.read()

    config_yaml = Template(config_yaml).render(**env)
    config = yaml.load(config_yaml)

    return config


def _create_stages(stages):
    for name in stages:
        stage = stages[name]

        if not isinstance(stage, dict):
            abort('"{0}" is not a valid environment/stage'.format(name))

        stage = _normalize_config(stage)
        module_obj = sys.modules[__name__]
        setattr(module_obj, name, _create_stage(name, stage))


def _normalize_config(config):
    for key, value in config.iteritems():
        if key == 'host' and isinstance(value, six.string_types):
            value = [value]

        if key == 'compose_files' and isinstance(value, six.string_types):
            value = [value]

        if key == 'env_files' and isinstance(value, six.string_types):
            value = [value]

        config[key] = value

    return config


def _parse_settings(data):

    defaults = {
        'env_files_dir': 'docker-compose-config',
        'files_dir': 'docker-compose-files',
        'extensions': []
    }
    settings = {}
    settings.update(defaults)
    settings.update(data)

    return settings


def _load_custom_tasks(tasks):
    sys.path.append(os.getcwd())

    for task_path in tasks:
        name = task_path.split(".")[-1]
        try:
            globals()[name] = importlib.import_module(task_path)
        except ImportError as e:  # NOQA
            abort('Task "{0}" not found'.format(task_path))


if __name__ == 'main':
    # Load configuration
    search_files = ['skeppa.yml', 'skeppa.yaml']
    config_path = None

    for path in search_files:
        if os.path.exists(path):
            config_path = path
            break

    if not config_path:
        abort('Config file {0} was not found'.format(search_files[0]))

    config = _load_config(config_path)

    # Manage settings
    settings_data = config.pop('settings', {})

    instance = skeppa_settings.Settings(values=settings_data)
    skeppa_settings.use(instance)

    settings = skeppa_settings.get_settings()

    # Activate extensions
    for ext_name in settings.extensions:
        extension = importlib.import_module('ext.{0}'.format(ext_name))
        ext.register(extension.extension)

    # Load any extra tasks
    _load_custom_tasks(settings.tasks)

    # Create stages
    _create_stages(config)



from __future__ import absolute_import

import imp

from fabric.api import local, run
from fabric.utils import abort
from fabric.state import env

from . import Extension


class Ecr(Extension):
    def register(self):
        try:
            imp.find_module('awscli')
        except ImportError:
            abort('The ecr extension requires the awscli package '
                  '(hint: pip install awscli)')

    def _verify_image(self, image):
        repository = image.get('repository')
        repro_type = repository.get('type', None)
        return repro_type == 'ecr'

    @staticmethod
    def _build_cli_args(values):
        command = ""
        for key in values:
            if not values[key]:
                continue
            command += " --%s=%s" % (key, values[key])

        return command

    def before_push(self, image):
        if not self._verify_image(image):
            return

        repository = image.get('repository')
        release_tag = image.get('tag', 'latest')

        repository_url = repository.get('url')
        region = repository.get('aws_region', 'us-east-1')
        profile = repository.get('aws_local_profile', '')

        self._login(local, image, region=region, profile=profile)

        # Try to delete previous release_tag from ECR
        try:
            registry_id = repository_url.split(".")[0]
            repository_name = repository_url.split("/")[-1]

            delete_args = {
                "region": region,
                "profile": profile
            }

            delete_command = "aws ecr batch-delete-image --registry-id {0} \
                --repository-name {1} \
                --image-ids imageTag={2}".format(registry_id, repository_name,
                                                 release_tag)

            delete_command += self._build_cli_args(delete_args)
            local(delete_command)
        except:
            pass

    def before_deploy(self, image):
        if not self._verify_image(image):
            return

        repository = image.get('repository')
        region = repository.get('aws_region', 'us-east-1')
        profile = repository.get('aws_profile', '')

        self._login(run, image, region=region, profile=profile)

    def _login(self, method, image, region, profile=""):
        # Authenticate with ecr
        auth_args = {
            "region": region,
            "profile": profile
        }

        auth_command = "aws ecr get-login"

        for arg in auth_args:
            if not auth_args[arg]:
                continue
            auth_command += " --%s=%s" % (arg, auth_args[arg])

        method("$(%s)" % auth_command)


extension = Ecr
